package com.wdkl.ncs.android.component.home.activity

import android.content.Intent
import android.os.Build
import android.text.TextUtils
import android.util.Log
import android.view.View
import com.enation.javashop.android.jrouter.external.annotation.Router
import com.enation.javashop.net.engine.model.NetState
import com.wdkl.ncs.android.component.home.BuildConfig
import com.wdkl.ncs.android.component.home.R
import com.wdkl.ncs.android.component.home.databinding.ActivityAppUpdateBinding
import com.wdkl.ncs.android.component.home.helper.AppUpdateHelper
import com.wdkl.ncs.android.component.home.helper.HttpHelper
import com.wdkl.ncs.android.component.home.launch.HomeLaunch
import com.wdkl.ncs.android.component.home.service.TcpHandleService
import com.wdkl.ncs.android.component.nursehome.common.Constants
import com.wdkl.ncs.android.lib.base.BaseActivity
import com.wdkl.ncs.android.lib.utils.AppTool
import com.wdkl.ncs.android.lib.utils.showMessage
import com.wdkl.ncs.android.lib.vo.filter
import com.wdkl.ncs.android.middleware.api.UrlManager
import com.wdkl.ncs.android.middleware.logic.contract.home.AppUpdateContract
import com.wdkl.ncs.android.middleware.logic.presenter.home.AppUpdatePresenter
import com.wdkl.ncs.android.middleware.model.dos.AppVersionDO
import com.wdkl.ncs.android.middleware.utils.MessageEvent
import kotlinx.android.synthetic.main.activity_app_update.*
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode
import java.io.File

@Router(path = "/app/update")
class AppUpdateActivity :BaseActivity<AppUpdatePresenter, ActivityAppUpdateBinding>(), AppUpdateContract.View {
    private val TAG = "AppUpdateActivity"

    private val urlManager = UrlManager.build()

    override fun getLayId(): Int {
        return R.layout.activity_app_update
    }

    override fun bindDagger() {
        HomeLaunch.component.inject(this)
    }

    override fun init() {
        checkAppVersion()
    }

    override fun handleAppVersion(appInfo: AppVersionDO) {
        //if ("YUNPAI_H6".equals(Build.MODEL)) {
            try {
                //启动云派app升级监听服务
                val intent = Intent()
                intent.setClassName("com.wd.app", "com.wd.app.MyService")
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                    //android8.0以上通过startForegroundService启动service
                    startForegroundService(intent)
                } else {
                    startService(intent)
                }
                Constants.yunpai_plugin = true
            } catch (e: Exception) {
                e.printStackTrace()
            }
        //}

        showMessage("获取版本 ==> 当前版本: " + BuildConfig.VERSION_NAME + "_" + BuildConfig.VERSION_CODE + ", 服务器版本: " + appInfo.versionCode + "_" + appInfo.versionNo)
        if (BuildConfig.VERSION_CODE < appInfo.versionNo){
            if (!TextUtils.isEmpty(appInfo.appPath)) {
                downLoadAPK(urlManager.device_url + appInfo.appPath)
            } else {
                showMessage("下载路径异常")
                finish()
            }
        } else {
            //showMessage("本机已经是最新版本")
            activity_calling_bed_text_download.text = "本机已经是最新版本"
            activity_appupdate_dialog_progressview.visibility = View.GONE
            TcpHandleService.updateLastTime = System.currentTimeMillis() / 1000
            AppTool.Time.delay(1000){
                finish()
            }
        }
    }

    fun checkAppVersion() {
        if (Constants.partId != null) {
            presenter.getAppVersion(Constants.partId, 19)
        }
    }

    /**
     * 下载APK包
     */
    fun downLoadAPK(url: String) {
        Log.d(TAG, "downLoadAPK  url==$url")
        activity_appupdate_dialog_progressview.setCurProgress(0)
        HttpHelper.download(url, object : HttpHelper.DownloadListener {
            override fun onDownloadSuccess() {
                Log.d("download", "onDownloadSuccess==" + "成功")
                runOnUiThread {
                    activity_calling_bed_text_download.text = "正在安装，请稍后..."
                }

                startInstallApk()

                /*if ("rk3288".equals(Build.MODEL)) {
                    installApkForRk3288()
                } else if ("YUNPAI_H6".equals(Build.MODEL)) {
                    startInstallAppYunPai()
                } else {
                    startInstallApk()
                }*/
            }

            override fun onDownloading(progress: Int) {
                runOnUiThread {
                    activity_appupdate_dialog_progressview.setCurProgress(progress)
                }
            }

            override fun onDownloadFailed() {
                Log.d("download", "onDownloadFailed==" + "失败")
                finish()
            }
        })
    }

    fun startInstallApk() {
        Thread{
            AppUpdateHelper.updateApp(this, object : AppUpdateHelper.UpdateCallBack {
                override fun onFailed() {
                    runOnUiThread {
                        showMessage("升级失败")
                        finish()
                    }
                }

                override fun onSuccess() {
                    runOnUiThread {
                        //showMessage("升级成功")
                        finish()
                        //有root权限用这个
//                        android.os.Process.killProcess(android.os.Process.myPid())
//                        System.exit(0)
                    }
                }
            })
        }.start()
    }

    //云派8寸机app静默升级
    fun startInstallAppYunPai() {
        val apkAbsolutePath = AppUpdateHelper.FILE_APK_PATH + "/" + AppUpdateHelper.FILE_APK_NAME
        val action = "com.miki.slient.INSTALL_CUSTOMER_PACKAGES"
        val intent = Intent(action)
        intent.putExtra("apkPath", apkAbsolutePath)
        intent.putExtra("packageName", "com.miki.sensortest")
        sendBroadcast(intent)
    }

    fun installApkForRk3288() {
        val path = AppUpdateHelper.FILE_APK_PATH + "/" + AppUpdateHelper.FILE_APK_NAME
        if (File(path).exists()) {
            val intent = Intent("com.android.yf_slient_install")
            intent.putExtra("path", path)  //要静默安装的apk路径
            intent.putExtra("isboot", true)  //安装完成后是否启动
            sendBroadcast(intent)
        } else {
            showMessage("安装文件不存在，升级失败!")
            finish()
        }
    }

    override fun bindEvent() {
        //
    }

    override fun destory() {
        //
    }

    //数据加载错误
    override fun onError(message: String, type: Int) {
        showMessage("获取版本失败，请检查网络或服务器版本!")
        finish()
    }

    override fun onNoneNet() {
        showMessage("没有网络!")
        finish()
    }

    //数据加载完成
    override fun complete(message: String, type: Int) {
        //
    }

    //开始获取数据
    override fun start() {
        //
    }

    //网络监听
    override fun networkMonitor(state: NetState) {
        state.filter(onMobile = {

        },onWifi = {

        },offline = {

        })
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onMoonEvent(messageEvent: MessageEvent) {

    }


}