package com.wdkl.ncs.android.component.home.helper;

import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.support.v4.BuildConfig;
import android.support.v4.content.FileProvider;
import android.util.Log;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

public class AppUpdateHelper {
    private final static String TAG = "AppUpdate";

    /**
     * 下载的APK文件绝对路径
     */
    public static final String FILE_APK_PATH = Environment.getExternalStorageDirectory() + "/IcuVisit";
    /**
     * 下载的APK文件的文件名
     */
    public static final String FILE_APK_NAME = "IcuVisitApp.apk";

    public static Context mContext;

    public static void updateApp(Context context, UpdateCallBack callBack) {
        mContext = context;
        if (checkApkExit(context)) {
            Log.d(TAG, "文件存在");
        } else {
            Log.d(TAG, "文件不存在");
            if (callBack != null) {
                callBack.onFailed();
            }
            return;
        }

        String path = FILE_APK_PATH + "/" + FILE_APK_NAME;

//        if (!checkApkAvailable(context, path)) {
//            Log.d(TAG, "安装失败");
//            return;
//        }


//        if (installApp(context.getPackageName(), path)) {
        if (rootSilenceInstall(path)) {
            Log.d(TAG, "安装成功");
            if (callBack != null) {
                callBack.onSuccess();
            }
        } else {
            Log.d(TAG, "安装失败");
            if (callBack != null) {
                callBack.onFailed();
            }
        }

//        if (silentInstall(context, path)) {
//            Log.d(TAG, "app 安装成功");
//            if (callBack != null) {
//                callBack.onSuccess();
//            }
//        }

        /*if (installAPP(path)) {
            Log.d(TAG, "安装成功");
            if (callBack != null) {
                callBack.onSuccess();
            }
        }*/
    }


    public static boolean installAPP(String path) {

        Intent intent = new Intent(Intent.ACTION_VIEW);
        File apkFile = new File(path);
        // 由于没有在Activity环境下启动Activity,设置下面的标签
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        //Android 7.0以上要使用FileProvider
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            intent.setFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            //参数1 上下文, 参数2 Provider主机地址 和配置文件中保持一致   参数3  共享的文件
            Uri uri = FileProvider.getUriForFile(mContext, mContext.getPackageName()+".provider", apkFile);
            //添加这一句表示对目标应用临时授权该Uri所代表的文件
//                    intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            intent.setDataAndType(uri, "application/vnd.android.package-archive");
        } else {
            intent.setDataAndType(Uri.fromFile(apkFile), "application/vnd.android.package-archive");
        }
        mContext.startActivity(intent);
        return true;
    }


    private static boolean checkApkExit(Context context) {
        File file = new File(FILE_APK_PATH + "/" + FILE_APK_NAME);
        return file.exists();
    }

    private static boolean checkApkAvailable(Context context, String path) {
        try {
            PackageManager pm = context.getPackageManager();
            PackageInfo info = pm.getPackageArchiveInfo(path, 0);
            ApplicationInfo applicationInfo = info.applicationInfo;
            String newPkg = applicationInfo.packageName;
            String curPkg = context.getPackageName();
            Log.d(TAG, "new package: " + newPkg + ", cur package: " + curPkg);
            if (curPkg.equals(newPkg)) {
                return true;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return false;
    }

    public static boolean silentInstall(Context context, String apkPath) {
        PackageManager packageManager = context.getPackageManager();
        Class pmClz = packageManager.getClass();
        try {
            if (Build.VERSION.SDK_INT >= 21) {
                Log.d(TAG, "apk path: " + apkPath);
                Class aClass = Class.forName("android.app.PackageInstallObserver");
                Constructor constructor = aClass.getDeclaredConstructor();
                constructor.setAccessible(true);
                Object installObserver = constructor.newInstance();
                Method method = pmClz.getDeclaredMethod("installPackage", Uri.class, aClass, int.class, String.class);
                method.setAccessible(true);
                method.invoke(packageManager, Uri.fromFile(new File(apkPath)), installObserver, 2, null);
            } else {
                Method method = pmClz.getDeclaredMethod("installPackage", Uri.class, Class.forName("android.content.pm.IPackageInstallObserver"), int.class, String.class);
                method.setAccessible(true);
                method.invoke(packageManager, Uri.fromFile(new File(apkPath)), null, 2, null);
            }
            return true;
        } catch (Exception e) {
            Log.e(TAG, e.toString());
        }
        return false;
    }

    public static boolean installApp(String packageName, String apkPath) {
        Process process = null;
        BufferedReader successResult = null;
        BufferedReader errorResult = null;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder errorMsg = new StringBuilder();
        Log.e(TAG, "install package: " + packageName + ", apkPath: " + apkPath);
        try {
            process = new ProcessBuilder("pm", "install", "-i", packageName, "-r", apkPath).start();
            successResult = new BufferedReader(new InputStreamReader(process.getInputStream()));
            errorResult = new BufferedReader(new InputStreamReader(process.getErrorStream()));
            String s;
            while ((s = successResult.readLine()) != null) {
                successMsg.append(s);
            }
            while ((s = errorResult.readLine()) != null) {
                errorMsg.append(s);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (successResult != null) {
                    successResult.close();
                }
                if (errorResult != null) {
                    errorResult.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            if (process != null) {
                process.destroy();
            }
        }
        Log.e(TAG, "" + errorMsg.toString());
        //如果含有“success”认为安装成功
        return successMsg.toString().equalsIgnoreCase("success");
    }

    public static boolean rootSilenceInstall(String path) {
        Process process;
        PrintWriter printWriter;
        try {
            process = Runtime.getRuntime().exec("su");
            printWriter = new PrintWriter(process.getOutputStream());
            printWriter.println("pm install -r " + path);
            printWriter.flush();
            printWriter.close();
            int res = process.waitFor();
            Log.e(TAG, "silent install res: " + res);
            if (res == 0) {
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            Log.e(TAG, "rootSilenceInstall e:" + e.getMessage());
            return false;
        }
    }

    public static void reboot(Context context) {
        Intent intent = new Intent(Intent.ACTION_REBOOT);
        intent.putExtra("nowait", 1);
        intent.putExtra("interval", 1);
        intent.putExtra("window", 0);
        context.sendBroadcast(intent);
    }

    public interface UpdateCallBack {
        void onFailed();

        void onSuccess();
    }
}
