package com.wdkl.ncs.android.component.home.fragment

import android.os.Bundle
import android.os.CountDownTimer
import android.support.v4.app.Fragment
import android.view.LayoutInflater
import android.view.MotionEvent
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import com.enation.javashop.utils.base.tool.BaseToolActivity
import com.wdkl.ncs.android.component.home.util.RingPlayHelper
import com.wdkl.ncs.android.component.nursehome.common.Constants
import com.wdkl.ncs.android.lib.utils.showMessage
import com.wdkl.ncs.android.middleware.tcp.channel.DeviceChannel
import com.wdkl.ncs.android.middleware.tcp.channel.VideoUtil
import com.wdkl.ncs.android.middleware.tcp.channel.VoiceUtil
import com.wdkl.ncs.android.middleware.utils.MessageEvent
import org.greenrobot.eventbus.EventBus

abstract class BaseCallFragment: Fragment(), View.OnTouchListener {

    private var layout: View? = null

    protected lateinit var baseActivity: BaseToolActivity

    //通话状态：0-去电, 1-来电
    protected var callState : Int = 0
    protected var onlyAudio: Boolean = false
    //来电设备id
    protected var fromId: Int = -1
    protected var interactionId: Int? = -1
    protected var targetId: String? = null

    //计时器
    lateinit var countDownTimer: CountDownTimer

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        retainInstance = true
        callState = arguments.getInt("call_state")
        onlyAudio = arguments.getBoolean("audio_only")
        fromId = arguments.getInt("fromId")
        interactionId = arguments.getInt("interactionId")
        targetId = arguments.getString("targetId")
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        if (layout == null) {
            layout = inflater.inflate(getLayId(), null)
        }

        /**初始化宿主Activity*/
        baseActivity = getActivity() as BaseToolActivity

        return layout
    }

    override fun onViewCreated(view: View?, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        init()
        bindEvent()

        view!!.setOnTouchListener(this)
    }

    override fun onDestroyView() {
        super.onDestroyView()
        destroy()
    }

    override fun onStart() {
        EventBus.getDefault().register(this)
        super.onStart()
    }

    override fun onStop() {
        EventBus.getDefault().unregister(this)
        super.onStop()
    }

    override fun onTouch(v: View?, event: MotionEvent?): Boolean {
        return true
    }

    protected abstract fun getLayId(): Int

    protected abstract fun init()

    protected abstract fun bindEvent()

    protected abstract fun destroy()

    protected abstract fun callEnd(handoff: Boolean)

    //初始化计时器
    protected fun initCountDownTimer(view: TextView) {
        countDownTimer = object: CountDownTimer(30000, 1000) {
            override fun onTick(millisUntilFinished: Long) {
                if (view != null) {
                    val time = millisUntilFinished/1000
                    view.setText("倒计时: " + time + "秒")
                }
            }

            override fun onFinish() {
                //呼叫超时，返回到主界面
                RingPlayHelper.stopRingTone()
                showMessage("无人应答...")
                DeviceChannel.calling = false
                Constants.CALL_STATE = Constants.CALL_STANDBY
                VideoUtil.cancelVideoCall(Constants.deviceId, Constants.interactionId)
                callEnd(false)
            }
        }
    }

    //开始计时
    protected fun startTimer() {
        if (countDownTimer != null) {
            countDownTimer.start()
        }
    }

    //取消计时器
    protected fun cancelTimer() {
        if (countDownTimer != null) {
            countDownTimer.cancel()
        }
    }

    //返回主界面
    protected fun backToMain() {
        EventBus.getDefault().post(MessageEvent("back_to_main", Constants.BACK_TO_MAIN_MSG))
    }

}