package com.wdkl.ncs.android.component.home.service

import android.app.Service
import android.content.Intent
import android.os.Binder
import android.os.IBinder
import android.util.Log
import com.alibaba.fastjson.JSONObject
import com.google.gson.Gson
import com.wdkl.ncs.android.component.home.activity.*
import com.wdkl.ncs.android.component.home.util.AppUtils
import com.wdkl.ncs.android.component.home.util.Util
import com.wdkl.ncs.android.component.nursehome.common.Constants
import com.wdkl.ncs.android.lib.base.BaseApplication
import com.wdkl.ncs.android.lib.utils.AppTool
import com.wdkl.ncs.android.middleware.model.vo.InteractionVO
import com.wdkl.ncs.android.middleware.tcp.channel.DeviceChannel
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel
import com.wdkl.ncs.android.middleware.tcp.enums.TcpAction
import com.wdkl.ncs.android.middleware.tcp.enums.TcpType
import com.wdkl.ncs.android.middleware.utils.CommonUtils
import com.wdkl.ncs.android.middleware.utils.MessageEvent
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode


class TcpHandleService : Service(){
    val TAG = TcpHandleService.javaClass.canonicalName
    companion object instance {
        var updateLastTime : Long = System.currentTimeMillis() / 1000
    }

    override fun onBind(intent: Intent?): IBinder? {
        return ServiceBinder()
    }

    inner class ServiceBinder : Binder() {
        fun doThings() {
        }
    }

    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
        Log.i("TcpHandleService", "已经启动")

        return super.onStartCommand(intent, flags, startId)
    }

    override fun onCreate() {
        super.onCreate()
        EventBus.getDefault().register(this)
    }

    override fun onDestroy() {
        EventBus.getDefault().unregister(this)
        super.onDestroy()
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onMoonEvent(messageEvent: MessageEvent) {
        when(messageEvent.tag){
            //APP升级
            Constants.EVENT_APP_UPDATE->{
                while (DeviceChannel.calling) {    //通话中不处理，一直等待至结束
                    Thread.sleep(3000)
                }

                DeviceChannel.calling = true    //置于通话状态中，不可呼叫
                Util.wakeUpAndUnlock(this)
                var tcpModel = messageEvent.getMessage() as TcpModel
                if (tcpModel.action == TcpAction.DeviceAction.APP_UPDATE) {
                    if ((System.currentTimeMillis() / 1000) - updateLastTime > 10){   //大于10秒可继续升级
                        updateLastTime = System.currentTimeMillis() / 1000
                        var intent = Intent()
                        intent.setClass(this, AppUpdateActivity::class.java)
                        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                        startActivity(intent)
                    }
                }
            }

            Constants.EVENT_RESTART_APP -> {
                AppUtils.restartApp()
            }

            //服务器IP地址切换
            Constants.EVENT_SERVER_CHANGE -> {
                val tcpModel = messageEvent.getMessage() as TcpModel
                if (tcpModel.data != null) {
                    val json = JSONObject.parseObject(tcpModel.data.toString())
                    val serverIp = json.getString("server_ip")
                    val serverPort = json.getString("server_port")
                    //Log.e(TAG, "change server ip:$serverIp, server port:$serverPort")
                    CommonUtils.setUrl(BaseApplication.appContext, serverIp)
                    CommonUtils.setUrlPort(BaseApplication.appContext, serverPort)
                    AppTool.Time.delay(2000) {
                        AppUtils.restartApp()
                    }
                }
            }

            //网络断开，1000ms重连
            /*Constants.EVENT_TCP_BREAK->{
                var wakeLock = Util.wakeUpAndUnlock(this)
                Thread(Runnable {
                    Thread.sleep(1000)
                    wakeLock?.release()
                }).start()
                Log.w(TAG, "EVENT TCP BREAK")
            }*/
        }
    }
}