package com.wdkl.ncs.android.component.home.util

import android.annotation.SuppressLint
import android.app.KeyguardManager
import android.content.Context
import android.content.Context.KEYGUARD_SERVICE
import android.os.Build
import android.os.Environment
import android.os.PowerManager
import android.telephony.TelephonyManager
import android.util.Log
import com.enation.javashop.utils.base.tool.SystemTool
import com.google.common.base.Strings
import java.io.File
import java.io.FileInputStream
import java.io.FileOutputStream
import java.io.IOException
import java.lang.Exception
import java.util.*

object Util {

    var IMEI = ""
    var UUID_FILE_NAME = "wdkl_uuid.txt"

    /*
      激活获取IMEI用
    */
    @SuppressLint("MissingPermission", "PrivateApi")
    fun getIMEI(context: Context): String {
        if (Build.VERSION.SDK_INT >= 29) {
            val UUIDStr = readSD()
            if (UUIDStr == null || UUIDStr.isEmpty()) { //如果为空或者空字符串就生成UUID创建文件并写入UUID
                val uuid: String = Date().getTime().toString()
                writeSD(uuid)
                IMEI = uuid
            } else {
                IMEI = UUIDStr
            }
            return IMEI
        }

        IMEI = SystemTool.getPhoneImei(context)
        if (Strings.isNullOrEmpty(IMEI)) {
            val tm = (context.getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager)
            try {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                    IMEI = tm.imei
                } else {
                    IMEI = tm.deviceId
                }
            }catch (e:Exception){
            }

            if (Strings.isNullOrEmpty(IMEI)) {
                IMEI = SystemTool.getLocalMacAddress()
//                IMEI = "35" + //we make this look like a valid IMEI
//                        Build.BOARD.length % 10 + Build.BRAND.length % 10 + Build.CPU_ABI.length % 10 + Build.DEVICE.length % 10 + Build.DISPLAY.length % 10 + Build.HOST.length % 10 + Build.ID.length % 10 + Build.MANUFACTURER.length % 10 + Build.MODEL.length % 10 + Build.PRODUCT.length % 10 + Build.TAGS.length % 10 + Build.TYPE.length % 10 + Build.USER.length % 10 //13 digits
            }
        }
        return IMEI
    }

    /**
     * 亮屏并解锁
     */
    @SuppressLint("InvalidWakeLockTag")
    fun wakeUpAndUnlock(context: Context): PowerManager.WakeLock? {
        val pm = context.getSystemService(Context.POWER_SERVICE) as PowerManager
        var wakeLock: PowerManager.WakeLock? = null

        val screenOn = pm.isInteractive
        if (!screenOn) { //如果是熄灭状态
            wakeLock = pm.newWakeLock(PowerManager.ACQUIRE_CAUSES_WAKEUP or PowerManager.SCREEN_DIM_WAKE_LOCK, "TAG")
            wakeLock.acquire(3000)//亮屏
        }

        // 屏幕解锁
        val keyguardManager = context.getSystemService(KEYGUARD_SERVICE) as KeyguardManager
        val keyguardLock = keyguardManager.newKeyguardLock("unLock")
        // 屏幕锁定
        keyguardLock.reenableKeyguard()
        keyguardLock.disableKeyguard() // 解锁
        return wakeLock
    }

    @SuppressLint("InvalidWakeLockTag")
    fun getCpuWakeLock(context: Context): PowerManager.WakeLock {
        val pm = context.getSystemService(Context.POWER_SERVICE) as PowerManager
        val cpuWakeLock = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK or PowerManager.ON_AFTER_RELEASE,
                context.javaClass.canonicalName)
        return cpuWakeLock
    }

    fun appendSpace(para: String): String? {
        val length = para.length
        val value = CharArray(length shl 1)
        var i = 0
        var j = 0
        while (i < length) {
            value[j] = para[i]
            value[1 + j] = ' '
            ++i
            j = i shl 1
        }
        return String(value)
    }

    fun writeSD(content: String): String? {
        //文件输出流
        var out: FileOutputStream? = null
        //设置文件路径
        val file = File(Environment.getExternalStorageDirectory(), UUID_FILE_NAME)
        return try {
            out = FileOutputStream(file)
            out.write(content.toByteArray())
            content
        } catch (e: Exception) {
            e.printStackTrace()
            ""
        } finally {
            try {
                if (out != null) {
                    out.close()
                }
            } catch (e: IOException) {
                e.printStackTrace()
            }
        }
    }

    fun readSD(): String? {
        //文件输入流
        var `in`: FileInputStream? = null
        //设置文件路径
        val file = File(Environment.getExternalStorageDirectory(), UUID_FILE_NAME)
        return try {
            `in` = FileInputStream(file)
            //使用缓冲来读
            val buf = ByteArray(1024) //每1024字节读一次
            val builder = StringBuilder()
            while (`in`.read(buf) !== -1) {
                builder.append(String(buf).trim { it <= ' ' })
            }
            builder.toString()
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
            ""
        } finally {
            try {
                if (`in` != null) {
                    `in`.close()
                }
            } catch (e: IOException) {
                e.printStackTrace()
            }
        }
    }
}