package com.wdkl.ncs.android.middleware.tcp.channel;

import android.util.Log;

import com.google.gson.Gson;
import com.wdkl.ncs.android.component.nursehome.common.Constants;
import com.wdkl.ncs.android.middleware.model.vo.InteractionVO;
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel;
import com.wdkl.ncs.android.middleware.tcp.enums.TcpAction;
import com.wdkl.ncs.android.middleware.tcp.enums.TcpType;
import com.wdkl.ncs.android.middleware.utils.MessageEvent;

import org.greenrobot.eventbus.EventBus;

/**
 * 前提：所有安卓设备可以互相通话。
 * 在此前提下，所有安卓设备接收TCP信号的逻辑一致
 */
public class DeviceChannel {
    private static String TAG = DeviceChannel.class.getSimpleName();

    //通话中模拟
    public static boolean calling = false;

    public static TcpModel handleTcpReceived(TcpModel tcpModel){
        TcpModel responseTcpModel = null;
        Log.i(TAG,"收到tcp消息 DeviceChannel "+tcpModel.toJson());

        switch (tcpModel.getType()){
            case VOICE:
                if (tcpModel.getAction()==TcpAction.VoiceAction.SUCCESS) {
                    EventBus.getDefault().post(new MessageEvent(tcpModel, Constants.AUDIO_MSG));
                } else if(tcpModel.getAction() == TcpAction.VoiceAction.CALL){
                    if (calling) {
                        InteractionVO interactionVO = new Gson().fromJson(tcpModel.getData().toString(), InteractionVO.class);
                        responseTcpModel = VoiceUtil.voiceCalling(Constants.Companion.getDeviceId(), tcpModel.getFromId(), interactionVO.getId());
                        //给服务器发送正在通话中 tcp
                        return responseTcpModel;
                    } else { //得到通话
                        EventBus.getDefault().post(new MessageEvent(tcpModel, 1));
                    }
                } else if ((tcpModel.getAction() == TcpAction.VoiceAction.ACCEPT)  //我方呼出，对方接受
                        || (tcpModel.getAction() == TcpAction.VoiceAction.REJECT) //我方呼出，对方拒绝
                        || (tcpModel.getAction() == TcpAction.VoiceAction.CALLING) //我方呼出，对方通话中
                        || (tcpModel.getAction() == TcpAction.VoiceAction.HANDOFF)
                        || (tcpModel.getAction() == TcpAction.VoiceAction.FAILED) //我方呼出，对方不在线，设备离线或其它错误
                        || (tcpModel.getAction() == TcpAction.VoiceAction.CANCEL)) { //对方取消
                    if (tcpModel.getAction() == TcpAction.VoiceAction.CALLING){
                        final TcpModel tm = tcpModel;
                        new Thread(new Runnable() {
                            @Override
                            public void run() {
                                try {
                                    Thread.sleep(3000);
                                } catch (InterruptedException e) {
                                    e.printStackTrace();
                                }
                                EventBus.getDefault().post(new MessageEvent(tm, 2));
                            }
                        }).start();
                    } else {
                        EventBus.getDefault().post(new MessageEvent(tcpModel, 2));
                    }
                }
                break;
            case VIDEO:
                EventBus.getDefault().post(new MessageEvent(tcpModel, Constants.VIDEO_MSG));
                break;
            case DEVICE:
                if (tcpModel.getAction() == TcpAction.DeviceAction.RESTART) {
                    EventBus.getDefault().post(new MessageEvent(tcpModel, Constants.EVENT_DEVICE_CHANGE));
                } else if (tcpModel.getAction() == TcpAction.DeviceAction.USER_CHANGE){
                    EventBus.getDefault().post(new MessageEvent(tcpModel, Constants.EVENT_USER_CHANGE));
                } else if (tcpModel.getAction() == TcpAction.DeviceAction.DEVICE_REFRESH){
                    //todo: 未处理
                } else if (tcpModel.getAction() == TcpAction.DeviceAction.APP_UPDATE){
                    EventBus.getDefault().post(new MessageEvent(tcpModel, Constants.EVENT_APP_UPDATE));
                } else if (tcpModel.getAction() == TcpAction.DeviceAction.SERVER_CHANGE) {
                    EventBus.getDefault().post(new MessageEvent(tcpModel, Constants.EVENT_SERVER_CHANGE));
                }
                break;
            case CALLBACK:
            case BROADCAST:
                break;
            case IM:
                if (tcpModel.getAction() == TcpAction.IMAction.MSG){
                    EventBus.getDefault().post(new MessageEvent(tcpModel, 3));
                }
                break;
            case SOS:
                EventBus.getDefault().post(new MessageEvent(tcpModel, 999));
                break;
            case DATA:
            case TIME:
                EventBus.getDefault().post(new MessageEvent(tcpModel, Constants.TCP_MSG));
                break;
            case EVENT:
                if (tcpModel.getAction() == TcpAction.EventAction.KEY_CLICK){
                    EventBus.getDefault().post(new MessageEvent(tcpModel, 3));
                }
                break;
            default:
                EventBus.getDefault().post(new MessageEvent(tcpModel, 0));
        }

        return responseTcpModel;
    }
}
