package com.wdkl.ncs.android.middleware.tcp.enums;

import java.util.HashMap;
import java.util.Map;

/**
 * TCP传输类型的动作
 *
 * @author allen
 * 2021-03-30 15:30
 */
public interface TcpAction {
    String getName();
    String getDescription();

    enum CallbackAction implements TcpAction {
        SUCCESS("同步"),
        FAILED("失败");
        private String description;
        CallbackAction(String description){
            this.description = description;
        }
        public String getDescription() {
            return description;
        }

        public String getName(){
            return this.name();
        }

        private final static Map<String , CallbackAction> ENUM_MAP = new HashMap<String, CallbackAction>();
        static {
            for(CallbackAction v : values()) {
                ENUM_MAP.put(v.toString() , v);
            }
        }
        public static CallbackAction fromString(String v) {
            CallbackAction userOptionEnum = ENUM_MAP.get(v);
            return userOptionEnum == null ? SUCCESS :userOptionEnum;
        }
    }

    enum VoiceAction implements TcpAction {
        CALL("呼叫"),
        ACCEPT("接受呼叫"),
        REJECT("拒绝"),
        CALLING("通话中"),
        TRANSFER("转接"),
        HANDOFF("挂断"),
        CANCEL("取消"),
        SUCCESS("呼叫成功"),
        FAILED("呼叫失败");


        private String description;
        VoiceAction(String description){
            this.description = description;
        }

        public String getDescription() {
            return description;
        }

        public String getName(){
            return this.name();
        }

        private final static Map<String , VoiceAction> ENUM_MAP = new HashMap<String, VoiceAction>();
        static {
            for(VoiceAction v : values()) {
                ENUM_MAP.put(v.toString() , v);
            }
        }
        public static VoiceAction fromString(String v) {
            VoiceAction userOptionEnum = ENUM_MAP.get(v);
            return userOptionEnum == null ? CALL :userOptionEnum;
        }
    }

    enum VideoAction implements TcpAction {
        CALL("视频通话"),   //正常视频对讲
        VIDEO_OUT_CALL("我方视频输出"),   //我方视频输出（我方语音，对方可视）
        VIDEO_IN_CALL("对方视频输出"),    //对方视频输出（我方可视，对方语音）
        VIDEO_ON("打开视频"),   //打开视频
        VIDEO_INVITE_JOIN("邀请加入视频"),    //邀请加入视频
        ACCEPT("接受呼叫"),
        REJECT("拒绝"),
        CALLING("通话中"),
        HANDOFF("挂断"),
        CANCEL("取消"),
        SUCCESS("呼叫成功"),
        FAILED("呼叫失败");

        private String description;
        VideoAction(String description){
            this.description = description;
        }

        public String getDescription() {
            return description;
        }

        public String getName(){
            return this.name();
        }

        private final static Map<String , VideoAction> ENUM_MAP = new HashMap<String, VideoAction>();
        static {
            for(VideoAction v : values()) {
                ENUM_MAP.put(v.toString() , v);
            }
        }
        public static VideoAction fromString(String v) {
            VideoAction userOptionEnum = ENUM_MAP.get(v);
            return userOptionEnum == null ? CALL :userOptionEnum;
        }
    }


    enum SOSAction implements TcpAction {
        CALL("紧急呼叫"),
        CANCEL("取消");

        private String description;
        SOSAction(String description){
            this.description = description;
        }

        public String getDescription() {
            return description;
        }

        public String getName(){
            return this.name();
        }

        private final static Map<String , SOSAction> ENUM_MAP = new HashMap<String, SOSAction>();
        static {
            for(SOSAction v : values()) {
                ENUM_MAP.put(v.toString() , v);
            }
        }
        public static SOSAction fromString(String v) {
            SOSAction userOptionEnum = ENUM_MAP.get(v);
            return userOptionEnum == null ? CALL :userOptionEnum;
        }
    }


    enum IMAction implements TcpAction {
        MSG("语音留言"),
        MSG_READ("语音留言已读");

        private String description;
        IMAction(String description){
            this.description = description;
        }

        public String getDescription() {
            return description;
        }

        public String getName(){
            return this.name();
        }

        private final static Map<String , IMAction> ENUM_MAP = new HashMap<String, IMAction>();
        static {
            for(IMAction v : values()) {
                ENUM_MAP.put(v.toString() , v);
            }
        }
        public static IMAction fromString(String v) {
            IMAction userOptionEnum = ENUM_MAP.get(v);
            return userOptionEnum == null ? MSG_READ :userOptionEnum;
        }

    }

    enum DeviceAction implements TcpAction {
        RESTART("重启"),
        CONNECT("连接"),
        APP_UPDATE("APP更新"),
        DEVICE_REFRESH("设备刷新"),
        SYSTEM_SETTING("系统设置"),
        DEVICE_CHANGE("设备更换"),
        USER_CHANGE("用户绑定"),
        SERVER_CHANGE("设备ip地址更换");

        private String description;
        DeviceAction(String description){
            this.description = description;
        }
        public String getDescription() {
            return description;
        }

        public String getName(){
            return this.name();
        }

        private final static Map<String , DeviceAction> ENUM_MAP = new HashMap<String, DeviceAction>();
        static {
            for(DeviceAction v : values()) {
                ENUM_MAP.put(v.toString() , v);
            }
        }
        public static DeviceAction fromString(String v) {
            DeviceAction userOptionEnum = ENUM_MAP.get(v);
            return userOptionEnum == null ? RESTART :userOptionEnum;
        }
    }

    enum EventAction implements TcpAction {
        KEY_CLICK("按键事件"),
        RESPONSE("已响应");
        private String description;
        EventAction(String description){
            this.description = description;
        }
        public String getDescription() {
            return description;
        }

        public String getName(){
            return this.name();
        }

        private final static Map<String , EventAction> ENUM_MAP = new HashMap<String, EventAction>();
        static {
            for(EventAction v : values()) {
                ENUM_MAP.put(v.toString() , v);
            }
        }
        public static EventAction fromString(String v) {
            EventAction userOptionEnum = ENUM_MAP.get(v);
            return userOptionEnum == null ? KEY_CLICK :userOptionEnum;
        }
    }

    enum DataAction implements TcpAction {
        REFRESH("刷新数据"),
        INTERACTION("刷新交互列表");
        private String description;
        DataAction(String description){
            this.description = description;
        }
        public String getDescription() {
            return description;
        }

        public String getName(){
            return this.name();
        }

        private final static Map<String , DataAction> ENUM_MAP = new HashMap<String, DataAction>();
        static {
            for(DataAction v : values()) {
                ENUM_MAP.put(v.toString() , v);
            }
        }
        public static DataAction fromString(String v) {
            DataAction userOptionEnum = ENUM_MAP.get(v);
            return userOptionEnum == null ? REFRESH :userOptionEnum;
        }
    }

    enum BroadcastAction implements TcpAction {
        START("开始"),
        STOP("停止");
        private String description;
        BroadcastAction(String description){
            this.description = description;
        }
        public String getDescription() {
            return description;
        }

        public String getName(){
            return this.name();
        }

        private final static Map<String , BroadcastAction> ENUM_MAP = new HashMap<String, BroadcastAction>();
        static {
            for(BroadcastAction v : values()) {
                ENUM_MAP.put(v.toString() , v);
            }
        }
        public static BroadcastAction fromString(String v) {
            BroadcastAction userOptionEnum = ENUM_MAP.get(v);
            return userOptionEnum == null ? START :userOptionEnum;
        }
    }

    enum TimeAction implements TcpAction {
        SYNC("同步");
        private String description;
        TimeAction(String description){
            this.description = description;
        }
        public String getDescription() {
            return description;
        }

        public String getName(){
            return this.name();
        }

        private final static Map<String , TimeAction> ENUM_MAP = new HashMap<String, TimeAction>();
        static {
            for(TimeAction v : values()) {
                ENUM_MAP.put(v.toString() , v);
            }
        }
        public static TimeAction fromString(String v) {
            TimeAction userOptionEnum = ENUM_MAP.get(v);
            return userOptionEnum == null ? SYNC :userOptionEnum;
        }
    }
}
